<?php
/**
 * Pagadito.
 * 
 * Pagadito Payment Extension v2.0.0
 * 
 * Este programa es Software Libre: Usted puede redistribuirlo y/o modificarlo
 * bajo los terminos de la Licencia Publica General Reducida de GNU (GNU Lesser
 * Public Licence), tal como se encuentra publicada por la Free Software
 * Foundation, ya sea por su version 3 o cualquier otra version superior.
 * 
 * Este programa es distribuido en el espiritu de que sea util, pero SIN NINGUNA
 * GARANTIA; sin tampoco garantia implicita de MERCANTIBILIDAD o ADAPTABILIDAD
 * PARA UN USO PARTICULAR. Vea la licencia GNU LGPL para mayores detalles.
 * 
 * @package     pagadito-v2.0.0
 * @copyright   Copyright (c) 2015 - Pagadito, S. de R.L. (https://www.pagadito.com)
 * @license     http://www.gnu.org/licenses/lgpl.html
 * @author      Pagadito Development Team <developers@pagadito.com>
 */
class ControllerExtensionPaymentPagadito extends Controller {
    
    private $pg;

    public function index() {
        
        $this->language->load('payment/pagadito');
        
        $data['text_testmode'] = $this->language->get('text_testmode');
        $data['button_confirm'] = $this->language->get('button_confirm');
        $data['testmode'] = $this->config->get('pagadito_test');
        $data['text_loading'] = $this->language->get('text_loading');

        $this->load->model('checkout/order');
        $order_info = $this->model_checkout_order->getOrder($this->session->data['order_id']);
        
        if ($order_info) {
            $data['action'] = $this->url->link('extension/payment/pagadito/process');
            return $this->load->view('extension/payment/pagadito', $data);
        }
    }   

    public function process() {
        $this->load->model('checkout/order');
        $order_info = $this->model_checkout_order->getOrder($this->session->data['order_id']);

        if ($order_info) {
            $data['return'] = $this->url->link('checkout/success');
            $data['notify_url'] = $this->url->link('payment/pagadito/callback', '', 'SSL');
            $data['cancel_return'] = $this->url->link('checkout/checkout', '', 'SSL');

            $products = array();
            $currency_code = strtoupper($this->session->data['currency']);
            foreach ($this->cart->getProducts() as $product) {
                $products[] =
                        array(
                            "quantity" => $product["quantity"],
                            "description" => $product["name"] . ' ' . $product["model"],
                            "price" => $this->currency->format($product['price'], $order_info['currency_code'], false, false),
                            "url_product" => $this->url->link('product/product&product_id=' . $product["product_id"])
                );
            }
            /* PROCESANDO SHIPPING COMO NUEVO ITEM */
            if ($this->cart->hasShipping()) {
                $total = $this->currency->format($order_info['total'] - $this->cart->getSubTotal(), $order_info['currency_code'], false, false);
                if ($total > 0) {
                    $products[] =
                            array(
                                "quantity" => 1,
                                "description" => $order_info['shipping_method'],
                                "price" => $total,
                                "url_product" => ''
                    );
                }
            }

            /* PROCESANDO TAXES COMO NUEVO ITEM */
            $taxes = $this->cart->getTaxes();
            if (count($taxes) > 0) {
                foreach ($taxes as $tax_rate_id => $amount) {
                    if ($amount > 0) {
                        $taxtRate = $this->db->query("SELECT name FROM `" . DB_PREFIX . "tax_rate` where tax_rate_id='" . $tax_rate_id . "'");
                        $products[] =
                                array(
                                    "quantity" => 1,
                                    "description" => $taxtRate->row['name'],
                                    "price" => $amount,
                                    "url_product" => ''
                        );
                    }
                }
            }

            $this->openGateway();
            if ($this->pg->connect()) {
                /* GUARDANDO TOKEN EN SESION */
                $this->session->data['pg_token'] = $this->pg->get_rs_value();
                $this->log->write('TOKEN : ' . $this->pg->get_rs_value().', ORDER ID: '.$this->session->data['order_id']);
                /* ENVIANDO TRANSACCION EN DIFERENTE MONEDA */
                if ($currency_code == 'GTQ') {
                    $this->pg->change_currency_gtq();
                } elseif ($currency_code == 'HNL') {
                    $this->pg->change_currency_hnl();
                } elseif ($currency_code == 'NIO') {
                    $this->pg->change_currency_nio();
                } elseif ($currency_code == 'CRC') {
                    $this->pg->change_currency_crc();
                } elseif ($currency_code == 'PAB') {
                    $this->pg->change_currency_pab();
                } elseif ($currency_code == 'DOP') {
                    $this->pg->change_currency_dop();
                } else {
                    $this->pg->change_currency_usd();
                }
                foreach ($products as $product) {
                    $this->pg->add_detail($product["quantity"], $product["description"], $product["price"], $product["url_product"]);
                }
                if (!$this->pg->exec_trans($order_info['order_id'])) {
                    $this->log->write('PAGADITO :: TRANSACCTION ERROR! ' . $this->pg->get_rs_code() . ': ' . $this->pg->get_rs_message());
                    $this->response->redirect($this->url->link('checkout/checkout', '', 'SSL'));
                }
            } else {
                $this->log->write('PAGADITO :: NOT CONNECTION! ' . $this->pg->get_rs_code() . ': ' . $this->pg->get_rs_message());
                $this->response->redirect($this->url->link('checkout/checkout', '', 'SSL'));
            }
        } else {
            $this->response->redirect($this->url->link('checkout/checkout', '', 'SSL'));
        }
    }

    public function callback() {
        $this->language->load('payment/pagadito');
        $url_redirect = $this->url->link('checkout/checkout', '', 'SSL');
        if (!isset($this->session->data['pg_token']) && !isset($this->request->get['token'])) {
            $this->log->write('CHECKOUT PG:: TOKEN PG NOT FOUND! order_id:' . $this->session->data['order_id']);
            $this->showError($this->language->get('error_token'), $url_redirect);
        } else {
            $token = isset($this->request->get['token']) ? $this->request->get['token'] : $this->session->data['pg_token'];
            $order_id = $this->session->data['order_id'];
            $this->openGateway();
            $order_status_id = $this->config->get('config_order_status_id');
            $message = '';
            if (!$this->pg->connect()) {
                $this->log->write('PAGADITO :: NOT CONNECTION ! pg_token:' . $token . ', order_id:' . $order_id . ', pg_code:' . $this->pg->get_rs_code() . ', pg_message:' . $this->pg->get_rs_message());
                $this->showError($this->language->get('error_connection'), $url_redirect);
            } elseif (!$this->pg->get_status($token)) {
                $this->log->write('PAGADITO :: INCOMPLET! pg_token:' . $token . ', order_id:' . $order_id . ', pg_code:' . $this->pg->get_rs_code() . ', pg_message:' . $this->pg->get_rs_message() . ' get_rs_status:' . $this->pg->get_rs_status() . ' reference:' . $this->pg->get_rs_reference());
                $this->showError($this->language->get('error_incomplet'), $url_redirect);
            } else {
                $url_redirect = $this->url->link('account/order', '', 'SSL');
                /* CHECK STATUS */
                $transaction_status = $this->pg->get_rs_status();
                $transaction_reference = $this->pg->get_rs_reference();
                $transaction_code = $this->pg->get_rs_code();
                $transaction_message = $this->pg->get_rs_message();
                $clear_cart = FALSE;
                switch ($transaction_status) {
                    case "COMPLETED":
                        $order_status_id = $this->config->get('pagadito_completed_status_id');
                        $message = 'COMPROBANTE DE PAGO : ' . $transaction_reference;
                        $url_redirect = $this->url->link('checkout/success', '', 'SSL');
                        break;
                    case "REGISTERED":
                        $order_status_id = $this->config->get('pagadito_processed_status_id');
                        $message = $this->language->get('error_registered');
                        break;
                    case "CANCELED":
                        $order_status_id = $this->config->get('pagadito_canceled_status_id');
                        $message = $this->language->get('error_canceled');
                        break;
                    case "FAILED":
                        $clear_cart = TRUE;
                        $order_status_id = $this->config->get('pagadito_failed_status_id');
                        $message = $this->language->get('error_failed') . '<br/>IDT PAGADITO : ' . $transaction_reference;
                        break;
                }
                $this->log->write('PAGADITO :: STATUS pg_token:' . $token . ', order_id:' . $order_id . ', pg_code:' . $transaction_code . ', pg_message:' . $transaction_message . ' get_rs_status:' . $transaction_status . ' reference:' . $transaction_reference);
                $this->load->model('payment/pagadito');
                $this->load->model('checkout/order');
                $order_info = $this->model_checkout_order->getOrder($order_id);
                if (!$order_info) {
                    $this->log->write('CHECKOUT PG:: ORDER NOT FOUND! order_id:' . $order_id . ' ' . $message);
                    $this->showError($this->language->get('error_order') . '<br/>' . $message, $url_redirect);
                } else {
                    if (!$order_info['order_status_id']) {
                        $this->model_checkout_order->addOrderHistory($order_id, $order_status_id, $message, TRUE);
                    } else {
                        $this->model_checkout_order->addOrderHistory($order_id, $order_status_id, $message, TRUE);
                    }
                    $this->response->redirect($url_redirect);
                }
            }
        }
    }

    private function openGateway() {
        $this->load->library('Pagadito');
        $this->pg = new Pagadito($this->config->get('pagadito_uid'), $this->config->get('pagadito_wsk'));
        if ($this->config->get('pagadito_test')) {
            $this->pg->mode_sandbox_on();
        }
    }

    private function showError($text_error, $link) {
        $data['text_error'] = $text_error;

        $data['button_continue'] = $this->language->get('button_continue');

        $data['continue'] = $link;
        $this->language->load('checkout/cart');
        $cart = $this->language->get('heading_title');
        $this->language->load('payment/pagadito');
        $data['heading_title'] = $this->language->get('heading_title');
        $this->language->load('checkout/checkout');
        $checkout = $this->language->get('heading_title');


        $data['breadcrumbs'] = array(
            array(
                'href' => $this->url->link('common/home'),
                'text' => $this->language->get('text_home'),
                'separator' => false
            ), array(
                'href' => $this->url->link('checkout/cart'),
                'text' => $cart,
                'separator' => $this->language->get('text_separator')
            ),
            array(
                'href' => $this->url->link('checkout/checkout'),
                'text' => $checkout,
                'separator' => $this->language->get('text_separator')
            )
        );

        $this->document->setTitle($this->language->get('heading_title'));
        unset($this->session->data['success']);
        
        $data['column_left'] = $this->load->controller('common/column_left');
        $data['column_right'] = $this->load->controller('common/column_right');
        $data['content_top'] = $this->load->controller('common/content_top');
        $data['content_bottom'] = $this->load->controller('common/content_bottom');
        $data['footer'] = $this->load->controller('common/footer');
        $data['header'] = $this->load->controller('common/header');
        
        if (file_exists(DIR_TEMPLATE . $this->config->get('config_template') . '/template/error/not_found.tpl')) {
            
            $this->response->setOutput($this->load->view($this->config->get('config_template') . '/template/error/not_found.tpl', $data));
        } else {
            
            $this->response->setOutput($this->load->view('default/template/error/not_found.tpl', $data));
        }
    }

    private function clearCart() {
        if (isset($this->session->data['order_id'])) {
            $this->cart->clear();

            unset($this->session->data['shipping_method']);
            unset($this->session->data['shipping_methods']);
            unset($this->session->data['payment_method']);
            unset($this->session->data['payment_methods']);
            unset($this->session->data['guest']);
            unset($this->session->data['comment']);
            unset($this->session->data['order_id']);
            unset($this->session->data['coupon']);
            unset($this->session->data['reward']);
            unset($this->session->data['voucher']);
            unset($this->session->data['vouchers']);
        }
    }

}

?>
